/**
 * Copyright 2012-2013 Peergreen S.A.S.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.ow2.util.osgi.toolkit.filter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.osgi.framework.Constants;

/**
 * A {@code Filters} is ...
 *
 * @author Guillaume Sauthier
 */
public class Filters {

    public static IFilter filter(final String expression) {
        return new Filter(expression);
    }

    public static IFilter present(final String name) {
        return new Filter(name + "=*");
    }

    public static IFilter equal(final String name, final String value) {
        return new Filter(name, Operand.EQUAL, value);
    }

    public static IFilter and(final IFilter first, final IFilter... chain) {
        if (first == null) {
            throw new IllegalArgumentException("LDAP & operand requires multiple arguments");
        }
        // Optimize
        if (chain == null || chain.length == 0) {
            return first;
        }
        List<IFilter> and = new ArrayList<IFilter>();
        and.add(first);
        and.addAll(Arrays.asList(chain));
        return new AndFilter(and);
    }

    public static IFilter or(final IFilter first, final IFilter... chain) {
        if (first == null) {
            throw new IllegalArgumentException("LDAP | operand requires multiple arguments");
        }
        // Optimize
        if (chain == null || chain.length == 0) {
            return first;
        }
        List<IFilter> or = new ArrayList<IFilter>();
        or.add(first);
        or.addAll(Arrays.asList(chain));
        return new OrFilter(or);
    }

    public static IFilter not(final IFilter filter) {
        return new NotFilter(filter);
    }

    public static IFilter objectClass(final String type) {
        return new Filter(Constants.OBJECTCLASS, Operand.EQUAL, type);
    }

    public static IFilter serviceId(final Long serviceId) {
        return serviceId(Operand.EQUAL, serviceId);
    }

    public static IFilter serviceId(final Operand operand, final Long serviceId) {
        return new Filter(Constants.SERVICE_ID, operand, String.valueOf(serviceId));
    }

    public static IFilter servicePid(final String servicePid) {
        return new Filter(Constants.SERVICE_PID, Operand.EQUAL, servicePid);
    }
}
